class Number : Object {

    private uint8[] _data;

    public Number(string value){
        _data = value.data;

        for (int i = 0; i < _data.length; i++){
            _data[i] -= 48;
        }
    }

    public Number.from_int(int value){
        this(value.to_string());
    }

    public Number.from_long(long value){
        this(value.to_string());
    }

    public uint8[] data {
        get {
            return _data;
        }
    }

    public int compare_to(Number n){
        if (_data.length > n.data.length){
            return 1;
        } else if (_data.length < n.data.length){
            return -1;
        }

        int i = 0;
        while (_data[i] == n.data[i] && i < _data.length){
            i++;
        }

        if (i == _data.length){
            return 0;
        } else if (_data[i] > n.data[i]){
            return 1;
        }
        return -1;
    }

    public void add(Number n){
        fill(_data.length + 1);
        n.fill(_data.length);

        for (int i = _data.length - 1; i > 0; i--){
            _data[i] = (_data[i] + n.data[i]) % 10;

            if (_data[i] < n.data[i]){
                int j = i - 1;

                while (_data[j] == 9){
                    _data[j] = 0;
                    j--;
                }
                _data[j]++;
            }
        }
        shrink();
        n.shrink();
    }

    public void multiply(Number n){
        Number i = new Number.from_int(1);
        Number c = clone();

        while (i.compare_to(n) < 0){
            add(c);
            i.add(new Number.from_int(1));
        }
    }

    public void factorial(){
        Number i = new Number.from_int(1);
        Number c = clone();

        while (i.compare_to(c) < 0){
            multiply(i);
            i.add(new Number.from_int(1));
        }
    }

    public void pow(Number n){
        Number i = new Number.from_int(1);
        Number c = clone();

        while (i.compare_to(n) < 0){
            multiply(c);
            i.add(new Number.from_int(1));
        }
    }

    public int digit_sum(){
        int sum = 0;

        for (int i = 0; i < _data.length; i++){
            sum += _data[i];
        }
        return sum;
    }

    /*private void dump(){
    	foreach (uint8 byte in _data){
    	    stdout.printf("%2.2x ", byte);
    	}
    	stdout.printf("\n");
    }*/

    private void fill(int size){
    	int old_length = _data.length;

        _data.resize(size);
        _data.move(0, size - old_length, size - 1);
    }

    private void shrink(){
        int k = 0;

        while (_data[k] == 0 && k < _data.length - 1){
            k++;
        }

        if (k == 0){
            return;
        }
        _data.move(k, 0, _data.length - k);
        _data.resize(_data.length - k);
    }

    public Number clone(){
        return new Number(to_string());
    }

    public string to_string(){
        uint8[] str = new uint8[_data.length];

        for (int i = 0; i < str.length; i++){
            str[i] = _data[i] + 48;
        }
        return (string)str;
    }
}
